#ifndef R_VERSION_H
#define R_VERSION_H 1
#define R_MESON_VERSION "1.5.2"
#define R2_VERSION_MAJOR 5
#define R2_VERSION_MINOR 9
#define R2_VERSION_PATCH 4
#define R2_VERSION_NUMBER 50904
#define R2_VERSION_COMMIT 0
#define R2_VERSION "5.9.4"
#define R2_GITTAP "5.9.4"
#define R2_GITTIP "unknown"
#define R2_BIRTH "2024-09-28__14:29:32"
#endif
